<?php

declare(strict_types=1);

namespace Drupal\partytown_drupal\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configure Partytown Drupal settings for this site.
 */
final class PartytownDrupalSettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'partytown_drupal_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['partytown_drupal.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $form['enable'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable Partytown'),
      '#default_value' => $this->config('partytown_drupal.settings')->get('enable'),
      '#description' => $this->t('Enable the Partytown library and attach it to the page.'),
    ];

    $form['debug'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Use Debug Version'),
      '#default_value' => $this->config('partytown_drupal.settings')->get('debug'),
      '#description' => $this->t('When true, Partytown scripts are not inlined and not minified.
        <br>
        See the <a href="https://partytown.qwik.dev/debugging/" target="_blank">Debugging</a>
        docs on how to enable more logging.'),
    ];

    $form['lib'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Lib Path'),
      '#default_value' => $this->config('partytown_drupal.settings')->get('lib'),
      '#description' => $this->t('The path to the partytown lib directory,
        with leading and trailing slashes.
       Default is <code>/modules/contrib/partytown_drupal/lib/</code>'),
    ];

    $form['forward'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Forward'),
      '#description' => $this->t('A list of strings representing
        function calls on the main thread to forward to the web worker.<br>
        See <a href="https://partytown.qwik.dev/forwarding-events/" target="_blank">
        Forwarding Events and Triggers</a> for more info.
        <br><br>
        Separate each forward call with a newline.'),
      '#default_value' => $this->config('partytown_drupal.settings')->get('forward'),
    ];

    $form['load_scripts_on_main_thread'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Load Scripts On Main Thread'),
      '#description' => $this->t('An array of strings or regular expressions
        (RegExp) used to filter out which script are executed via Partytown
        and the main thread.<br>
        <br>
        An example is as follows:<br>
        <code>https://test.com/analytics.js</code><br>
        <code>inline-script-id</code><br>
        <code>/regex-matched-script\.js/</code><br>
        <br>
        Separate each rule with a newline.'),
      '#default_value' => $this->config('partytown_drupal.settings')->get('load_scripts_on_main_thread'),
    ];

    $form['resolve_url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Resolve URL'),
      '#description' => $this->t('Javascript hook that is called to resolve
        URLs which can be used to modify URLs.<br>
        <br>
        The hook uses the API:
        <br>
        <code>resolveUrl(url: URL, location: URL, method: string)</code>.<br>
        See the <a href="https://partytown.qwik.dev/proxying-requests/" target="_blank">Proxying Requests</a>
        for more information.'),
      '#default_value' => $this->config('partytown_drupal.settings')->get('resolve_url'),
    ];

    $form['fallback_timeout'] = [
      '#type' => 'number',
      '#title' => $this->t('Fallback Timeout'),
      '#description' => $this->t('A timeout in ms until Partytown initialization is
        considered as failed & fallbacks to the regular execution in main thread.
        Default is 9999'),
      '#default_value' => $this->config('partytown_drupal.settings')->get('fallback_timeout'),
      '#min' => 0,
      '#max' => 1000000,
      '#step' => 1,
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $this->config('partytown_drupal.settings')
      ->set('enable', $form_state->getValue('enable'))
      ->set('debug', $form_state->getValue('debug'))
      ->set('lib', $form_state->getValue('lib'))
      ->set('forward', $form_state->getValue('forward'))
      ->set('load_scripts_on_main_thread', $form_state->getValue('load_scripts_on_main_thread'))
      ->set('resolve_url', $form_state->getValue('resolve_url'))
      ->set('fallback_timeout', $form_state->getValue('fallback_timeout'))
      ->save();
    parent::submitForm($form, $form_state);
  }

}
